@@ -1,8 +1,7 @@
-/*  $Id: Command.h,v 1.2 2007/06/26 02:55:04 sarrazip Exp $
-    Command.h - Game script command definition
+/*  $Id: Command.h,v 1.3 2011/10/31 01:37:53 sarrazip Exp $
 
     cosmosmash - A space rock shooting video game.
-    Copyright (C) 2007 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2007-2011 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -14,10 +13,10 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-    02111-1307, USA.
+    You should have received a copy of the GNU General Public
+    License along with this program; if not, write to the Free
+    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA  02110-1301, USA.
 */
 
 #ifndef _H_Command
