@@ -1,7 +1,5 @@
-/*  $Id: CosmoSmashEngine.cpp,v 1.70 2017/12/09 18:20:43 sarrazip Exp $
-
-    cosmosmash - A space rock shooting video game.
-    Copyright (C) 2000-2011 Pierre Sarrazin <http://sarrazip.com/>
+/*  cosmosmash - A space rock shooting video game.
+    Copyright (C) 2000-2024 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -40,42 +38,6 @@
 #include <iomanip>
 #include <sstream>
 
-#include "base.xpm"
-#include "boosted-base.xpm"
-#include "basebullet.xpm"
-#include "bigrock0.xpm"
-#include "bigrock1.xpm"
-#include "bigrock2.xpm"
-#include "bigrock3.xpm"
-#include "bigrock4.xpm"
-#include "bigrock5.xpm"
-#include "smallrock0.xpm"
-#include "smallrock1.xpm"
-#include "smallrock2.xpm"
-#include "smallrock3.xpm"
-#include "smallrock4.xpm"
-#include "smallrock5.xpm"
-#include "explosion0.xpm"
-#include "explosion1.xpm"
-#include "bigspinner_backslash.xpm"
-#include "bigspinner_horizontal.xpm"
-#include "bigspinner_slash.xpm"
-#include "bigspinner_vertical.xpm"
-#include "smallspinner_backslash.xpm"
-#include "smallspinner_horizontal.xpm"
-#include "smallspinner_slash.xpm"
-#include "smallspinner_vertical.xpm"
-#include "pulsar_empty.xpm"
-#include "pulsar_mid.xpm"
-#include "pulsar_full.xpm"
-#include "saucer0.xpm"
-#include "saucer1.xpm"
-#include "saucerbullet_empty.xpm"
-#include "saucerbullet_mid.xpm"
-#include "saucerbullet_full.xpm"
-#include "question0.xpm"
-#include "question1.xpm"
-
 using namespace std;
 using namespace flatzebra;
 
@@ -87,8 +49,7 @@ using namespace flatzebra;
 /*  Various constants:
 */
 
-static const size_t
-FRAMES_PER_SECOND = 20;
+static const size_t FRAMES_PER_SECOND = 20;
 
 static const Couple theDrawingPixmapSize(WINDOW_WIDTH, WINDOW_HEIGHT);
 
@@ -252,8 +213,9 @@ CosmoSmashEngine::CosmoSmashEngine(
                     bool _mirrorHyperspace, bool _useSound,
                     bool fullScreen, Controller *_controller,
                     ostream *_gameScriptStream,
-                    bool processActiveEvent)
-  : GameEngine(theDrawingPixmapSize, "Cosmosmash", fullScreen, processActiveEvent),
+                    bool processActiveEvent,
+                    bool useAcceleratedRendering)
+  : GameEngine(theDrawingPixmapSize, "Cosmosmash", fullScreen, processActiveEvent, useAcceleratedRendering),
                   // may throw string exception
 
     pauseState(PLAYING),
@@ -261,9 +223,9 @@ CosmoSmashEngine::CosmoSmashEngine(
     useSound(_useSound),
 
     levelColors(),
-    whiteColor(SDL_MapRGB(theSDLScreen->format, 0xff, 0xff, 0xff)),
-    greenColor(SDL_MapRGB(theSDLScreen->format, 0x00, 0xff, 0x00)),
-    explosionBGColor(SDL_MapRGB(theSDLScreen->format, 0xff, 0x80, 0x00)),
+    whiteColor(mapRGBA(0xff, 0xff, 0xff)),
+    greenColor(mapRGBA(0x00, 0xff, 0x00)),
+    explosionBGColor(mapRGBA(0xff, 0x80, 0x00)),
 
     mirrorHyperspace(_mirrorHyperspace),
     useGameExtensions(false),
@@ -325,7 +287,7 @@ CosmoSmashEngine::CosmoSmashEngine(
     starPositions(),
     mountainRangePositions(),
 
-    theSoundMixer(NULL),
+    soundEngine(NULL),
     playerHitSound(),
     pulsarBeepSound(),
     saucerShootingSound(),
@@ -336,17 +298,15 @@ CosmoSmashEngine::CosmoSmashEngine(
 {
     assert(controller != NULL);
 
-    levelColors[0] = 0;  // unused
-    levelColors[1] = SDL_MapRGB(theSDLScreen->format, 0x00, 0x00, 0x00);
-    levelColors[2] = SDL_MapRGB(theSDLScreen->format, 0x00, 0x00, 0xff);
-    levelColors[3] = SDL_MapRGB(theSDLScreen->format, 0x80, 0x00, 0x80);
-    levelColors[4] = SDL_MapRGB(theSDLScreen->format, 0x00, 0x7f, 0x7f);
-    levelColors[5] = SDL_MapRGB(theSDLScreen->format, 0x7f, 0x7f, 0x7f);
-    levelColors[6] = SDL_MapRGB(theSDLScreen->format, 0x00, 0x00, 0x00);
+    levelColors[0] = mapRGBA(0x00, 0x00, 0x00);  // unused
+    levelColors[1] = mapRGBA(0x00, 0x00, 0x00);
+    levelColors[2] = mapRGBA(0x00, 0x00, 0xff);
+    levelColors[3] = mapRGBA(0x80, 0x00, 0x80);
+    levelColors[4] = mapRGBA(0x00, 0x7f, 0x7f);
+    levelColors[5] = mapRGBA(0x7f, 0x7f, 0x7f);
+    levelColors[6] = mapRGBA(0x00, 0x00, 0x00);
 
     assert(initScore >= 0);
-    assert(theSDLScreen != NULL);
-    assert(theSDLScreen->format != NULL);
 
     try
     {
@@ -368,14 +328,25 @@ CosmoSmashEngine::~CosmoSmashEngine()
     killAllAutomaticCharacters();
     killSpritesInList(baseBulletSprites);
 
-    roundbeetle::Engine::destroy();
-    theSoundMixer = NULL;
+    if (useSound)
+        roundbeetle::SoundEngine::destroy();
+    soundEngine = NULL;
 
     delete controller;
     delete playerSprite;
 }
 
 
+void
+CosmoSmashEngine::loadPixmap(const char *filePath,
+                             PixmapArray &pa,
+                             size_t index)
+{
+    static const string dir = getDirPathFromEnv(PKGPIXMAPDIR, "PKGPIXMAPDIR");
+    GameEngine::loadPixmap(dir + filePath, pa, index);
+}
+
+
 void
 CosmoSmashEngine::initializeSprites()
 {
@@ -383,8 +354,8 @@ CosmoSmashEngine::initializeSprites()
     groundHeight = 10;
 
     // Load some pixmaps:
-    loadPixmap(base_xpm, playerPA, 0);
-    loadPixmap(boosted_base_xpm, playerPA, 1);
+    loadPixmap("base.xpm", playerPA, 0);
+    loadPixmap("boosted-base.xpm", playerPA, 1);
 
     // Create and position the player sprite:
     playerSprite = new Sprite(playerPA,
@@ -396,55 +367,55 @@ CosmoSmashEngine::initializeSprites()
 
 
     // Base bullets:
-    loadPixmap(basebullet_xpm, baseBulletPA, 0);
+    loadPixmap("basebullet.xpm", baseBulletPA, 0);
 
     // Rocks:
-    loadPixmap(bigrock0_xpm, bigRockPA, 0);
-    loadPixmap(bigrock1_xpm, bigRockPA, 1);
-    loadPixmap(bigrock2_xpm, bigRockPA, 2);
-    loadPixmap(bigrock3_xpm, bigRockPA, 3);
-    loadPixmap(bigrock4_xpm, bigRockPA, 4);
-    loadPixmap(bigrock5_xpm, bigRockPA, 5);
-    loadPixmap(smallrock0_xpm, smallRockPA, 0);
-    loadPixmap(smallrock1_xpm, smallRockPA, 1);
-    loadPixmap(smallrock2_xpm, smallRockPA, 2);
-    loadPixmap(smallrock3_xpm, smallRockPA, 3);
-    loadPixmap(smallrock4_xpm, smallRockPA, 4);
-    loadPixmap(smallrock5_xpm, smallRockPA, 5);
+    loadPixmap("bigrock0.xpm", bigRockPA, 0);
+    loadPixmap("bigrock1.xpm", bigRockPA, 1);
+    loadPixmap("bigrock2.xpm", bigRockPA, 2);
+    loadPixmap("bigrock3.xpm", bigRockPA, 3);
+    loadPixmap("bigrock4.xpm", bigRockPA, 4);
+    loadPixmap("bigrock5.xpm", bigRockPA, 5);
+    loadPixmap("smallrock0.xpm", smallRockPA, 0);
+    loadPixmap("smallrock1.xpm", smallRockPA, 1);
+    loadPixmap("smallrock2.xpm", smallRockPA, 2);
+    loadPixmap("smallrock3.xpm", smallRockPA, 3);
+    loadPixmap("smallrock4.xpm", smallRockPA, 4);
+    loadPixmap("smallrock5.xpm", smallRockPA, 5);
 
     // Big spinner:
-    loadPixmap(bigspinner_horizontal_xpm, bigSpinnerPA, 0);
-    loadPixmap(bigspinner_slash_xpm,      bigSpinnerPA, 1);
-    loadPixmap(bigspinner_vertical_xpm,   bigSpinnerPA, 2);
-    loadPixmap(bigspinner_backslash_xpm,  bigSpinnerPA, 3);
+    loadPixmap("bigspinner_horizontal.xpm", bigSpinnerPA, 0);
+    loadPixmap("bigspinner_slash.xpm",      bigSpinnerPA, 1);
+    loadPixmap("bigspinner_vertical.xpm",   bigSpinnerPA, 2);
+    loadPixmap("bigspinner_backslash.xpm",  bigSpinnerPA, 3);
 
     // Small spinner:
-    loadPixmap(smallspinner_horizontal_xpm, smallSpinnerPA, 0);
-    loadPixmap(smallspinner_slash_xpm,      smallSpinnerPA, 1);
-    loadPixmap(smallspinner_vertical_xpm,   smallSpinnerPA, 2);
-    loadPixmap(smallspinner_backslash_xpm,  smallSpinnerPA, 3);
+    loadPixmap("smallspinner_horizontal.xpm", smallSpinnerPA, 0);
+    loadPixmap("smallspinner_slash.xpm",      smallSpinnerPA, 1);
+    loadPixmap("smallspinner_vertical.xpm",   smallSpinnerPA, 2);
+    loadPixmap("smallspinner_backslash.xpm",  smallSpinnerPA, 3);
 
     // Pulsar:
-    loadPixmap(pulsar_empty_xpm, pulsarPA, 0);
-    loadPixmap(pulsar_mid_xpm,   pulsarPA, 1);
-    loadPixmap(pulsar_full_xpm,  pulsarPA, 2);
+    loadPixmap("pulsar_empty.xpm", pulsarPA, 0);
+    loadPixmap("pulsar_mid.xpm",   pulsarPA, 1);
+    loadPixmap("pulsar_full.xpm",  pulsarPA, 2);
 
     // Saucer:
-    loadPixmap(saucer0_xpm, saucerPA, 0);
-    loadPixmap(saucer1_xpm, saucerPA, 1);
+    loadPixmap("saucer0.xpm", saucerPA, 0);
+    loadPixmap("saucer1.xpm", saucerPA, 1);
 
     // Saucer bullets:
-    loadPixmap(saucerbullet_empty_xpm, saucerBulletPA, 0);
-    loadPixmap(saucerbullet_mid_xpm,   saucerBulletPA, 1);
-    loadPixmap(saucerbullet_full_xpm,  saucerBulletPA, 2);
+    loadPixmap("saucerbullet_empty.xpm", saucerBulletPA, 0);
+    loadPixmap("saucerbullet_mid.xpm",   saucerBulletPA, 1);
+    loadPixmap("saucerbullet_full.xpm",  saucerBulletPA, 2);
 
     // Explosion:
-    loadPixmap(explosion0_xpm, explosionPA, 0);
-    loadPixmap(explosion1_xpm, explosionPA, 1);
+    loadPixmap("explosion0.xpm", explosionPA, 0);
+    loadPixmap("explosion1.xpm", explosionPA, 1);
 
     // Question (surprise) sprites:
-    loadPixmap(question0_xpm, questionPA, 0);
-    loadPixmap(question1_xpm, questionPA, 1);
+    loadPixmap("question0.xpm", questionPA, 0);
+    loadPixmap("question1.xpm", questionPA, 1);
 }
 
 
@@ -532,7 +503,7 @@ CosmoSmashEngine::initializeMisc(bool useSound)
         // to make sure the spinner square wave's frequency
         // decreases smoothly to the human ear.
         //
-        int status = roundbeetle::Engine::create(44100, 0, NULL);
+        int status = roundbeetle::SoundEngine::create(44100, 0, NULL);
         if (status != 0)
         {
             stringstream ss;
@@ -540,9 +511,9 @@ CosmoSmashEngine::initializeMisc(bool useSound)
             throw ss.str();
         }
 
-        theSoundMixer = &roundbeetle::Engine::instance();
-        assert(theSoundMixer != NULL);
-        theSoundMixer->postBusInit();
+        soundEngine = &roundbeetle::SoundEngine::instance();
+        assert(soundEngine != NULL);
+        soundEngine->postBusInit();
 
         try
         {
@@ -599,7 +570,7 @@ CosmoSmashEngine::setTimeBeforeNextQuestion()
 
 /*virtual*/
 void
-CosmoSmashEngine::processKey(SDLKey keysym, bool pressed)
+CosmoSmashEngine::processKey(SDL_Keycode keysym, bool pressed)
 {
     controller->processKey(keysym, pressed);
 
@@ -650,8 +621,8 @@ void
 CosmoSmashEngine::pause()
 {
     registerCommand(COMMAND_PAUSE);
-    if (theSoundMixer != NULL)
-        (void) theSoundMixer->pauseEngine();
+    if (soundEngine != NULL)
+        (void) soundEngine->pauseEngine();
 }
 
 
@@ -659,8 +630,8 @@ void
 CosmoSmashEngine::resume()
 {
     registerCommand(COMMAND_RESUME);
-    if (theSoundMixer != NULL)
-        (void) theSoundMixer->resumeEngine();
+    if (soundEngine != NULL)
+        (void) soundEngine->resumeEngine();
 }
 
 
@@ -676,29 +647,29 @@ CosmoSmashEngine::tick()
     bool ret = true;
 
     if (controller->isFullScreenToggleRequested())
-        setVideoMode(theScreenSizeInPixels, !inFullScreenMode());  // ignore failure
+        setFullScreenMode(!inFullScreenMode());  // ignore failure
 
-    if (pauseState == AWAITING_RESUME)
+    if (pauseState == AWAITING_RESUME && controller->isResumeRequested())
     {
-        if (controller->isResumeRequested())
-        {
-            resume();
-            pauseState = PLAYING;
-        }
-        else
-            displayPauseMessage();
+        resume();
+        pauseState = PLAYING;
     }
     else
     {
-        tickCount++;
+        if (pauseState == PLAYING)
+        {
+            tickCount++;
 
-        detectCollisions();
+            detectCollisions();
+        }
 
-        if (!animatePlayer())
+        if (pauseState == PLAYING && !animatePlayer())
             ret = false;
         else
         {
-            animateAutomaticCharacters();
+            if (pauseState == PLAYING)
+                animateAutomaticCharacters();
+
             restoreBackground();
             drawSprites();
 
@@ -728,13 +699,15 @@ CosmoSmashEngine::tick()
                         finished exploding before displaying the start
                         message again.
                     */
-                    bool baseExploding = (playerSprite->getTimeToLive() != 0);
-                    if (! baseExploding)
+                    if (! isBaseExploding())
                         displayStartMessage(true);
                 }
             }
-            else
+            else if (pauseState == PLAYING)
                 playCadenceSound();
+
+            if (pauseState == AWAITING_RESUME)
+                displayPauseMessage();
         }
     }
 
@@ -1131,13 +1104,13 @@ CosmoSmashEngine::animateSaucers()
 void
 CosmoSmashEngine::startSpinnerSound(SpinnerSprite &s)
 {
-    if (theSoundMixer == NULL)
+    if (soundEngine == NULL)
         return;
     roundbeetle::ADSR adsr(.0f, .1f, 0, 0, 3600, 0);  // longer than needed
-    int req = theSoundMixer->requestSquareWave(s.getLinearMovingFreq(),
+    int req = soundEngine->requestSquareWave(s.getLinearMovingFreq(),
                                                adsr,
                                                1,
-                                               theSoundMixer->getMainBus());
+                                               soundEngine->getMainBus());
     s.setSoundReq(req);
 }
 
@@ -1175,7 +1148,7 @@ CosmoSmashEngine::animateAutomaticCharacters()
 
             SpinnerSprite *s = new SpinnerSprite(*pa, pos, speed,
                                                  size / 3, size / 3,
-                                                 theSoundMixer,
+                                                 soundEngine,
                                                  startFreq,
                                                  endFreq,
                                                  durationInS);
@@ -1404,6 +1377,12 @@ CosmoSmashEngine::animateAutomaticCharacters()
         assert(s->currentPixmapIndex <= 1);
     }
     removeNullElementsFromSpriteList(explosionSprites);
+
+
+    /*  Move the stars:
+    */
+    if (numLives > 0 && ! isBaseExploding() && tickCount % 4 == 0)
+        moveStars();
 }
 
 
@@ -1810,33 +1789,44 @@ CosmoSmashEngine::boostPlayer()
 ///////////////////////////////////////////////////////////////////////////////
 
 
+void
+CosmoSmashEngine::moveStars()
+{
+    CoupleList::iterator it;
+    for (it = starPositions.begin(); it != starPositions.end(); it++)
+    {
+        Couple &pos = *it;
+        pos.y--;
+        if (pos.y < 0)
+            pos.y = mountainTopPos - 1;
+    }
+}
+
+
+bool
+CosmoSmashEngine::isBaseExploding() const
+{
+    return (playerSprite->getTimeToLive() != 0);
+}
+
+
 void
 CosmoSmashEngine::restoreBackground()
 {
-    bool baseExploding = (playerSprite->getTimeToLive() != 0);
-
     // Background:
     assert(theLevel >= 1 && theLevel <= NUM_LEVELS);
-    Uint32 bgColor = (baseExploding ? explosionBGColor : levelColors[theLevel]);
+    SDL_Color bgColor = (isBaseExploding() ? explosionBGColor : levelColors[theLevel]);
     fillRect(0, 0, theDrawingPixmapSize.x, theDrawingPixmapSize.y, bgColor);
 
     // Ground:
     fillRect(0, groundPos, theDrawingPixmapSize.x, groundHeight, greenColor);
 
     // Stars:
-    bool moveStars = (numLives > 0 && ! baseExploding && tickCount % 4 == 0);
     CoupleList::iterator it;
     for (it = starPositions.begin(); it != starPositions.end(); it++)
     {
         Couple &pos = *it;
         drawPixel(pos.x, pos.y, whiteColor);
-
-        if (moveStars)
-        {
-            pos.y--;
-            if (pos.y < 0)
-                pos.y = mountainTopPos - 1;
-        }
     }
 
     // Mountain range:
@@ -1855,7 +1845,8 @@ CosmoSmashEngine::drawSprites()
 {
     if (playerIsActive())
     {
-        int pixmapIndex = (playerBoostTicks > 0 ? ((tickCount & 4) != 0) : 0);
+        size_t pixmapIndex = (playerBoostTicks > 0 ? ((tickCount & 4) != 0) : 0);
+        assert(playerSprite);
         copySpritePixmap(*playerSprite, pixmapIndex, playerSprite->getPos());
     }
 
@@ -2003,7 +1994,7 @@ CosmoSmashEngine::displayStartMessage(bool display)
         displayMessage(lineNum++, "Pause:              P                  ");
         displayMessage(lineNum++, "Toggle Full Screen: F11                ");
 
-        if (useSound && theSoundMixer == NULL)
+        if (useSound && soundEngine == NULL)
         {
             lineNum++;
             lineNum++;
@@ -2037,11 +2028,11 @@ CosmoSmashEngine::playShortExplosionSound()
 void
 CosmoSmashEngine::playSoundEffect(const roundbeetle::WaveFileBuffer &wfb)
 {
-    if (! playerIsActive() || theSoundMixer == NULL)
+    if (! playerIsActive() || soundEngine == NULL)
         return;
 
-    roundbeetle::Bus &mainBus = theSoundMixer->getMainBus();
-    int reqHandle = theSoundMixer->requestWaveFileSound(wfb, mainBus);
+    roundbeetle::Bus &mainBus = soundEngine->getMainBus();
+    int reqHandle = soundEngine->requestWaveFileSound(wfb, mainBus);
     if (reqHandle == -1)
         cout << PACKAGE ": failed to play sound " << &wfb << endl;
 }
@@ -2070,7 +2061,7 @@ CosmoSmashEngine::registerTickEnd()
 void
 CosmoSmashEngine::playCadenceSound()
 {
-    if (! playerIsActive() || theSoundMixer == NULL)
+    if (! playerIsActive() || soundEngine == NULL)
         return;
 
     if (timeBeforeNextCadenceSound > 0)
@@ -2101,7 +2092,7 @@ CosmoSmashEngine::playCadenceSound()
     float decayTime = cadenceInSeconds * .267f;
     float releaseTime = cadenceInSeconds * .4f;
     roundbeetle::ADSR adsr(.25f, .15f, .0f, decayTime, .0f, releaseTime);
-    /*int req =*/ theSoundMixer->requestWhiteNoise(adsr, 1, theSoundMixer->getMainBus());
+    /*int req =*/ soundEngine->requestWhiteNoise(adsr, 1, soundEngine->getMainBus());
     //if (req == -1)
     //    cout << PACKAGE ": failed to play cadence sound" << endl;
 }
