@@ -1,4 +1,4 @@
-/*  $Id: CosmoSmashEngine.h,v 1.47 2017/12/09 18:20:44 sarrazip Exp $
+/*  $Id: CosmoSmashEngine.h,v 1.50 2024/04/27 05:24:34 sarrazip Exp $
 
     cosmosmash - A space rock shooting video game.
     Copyright (C) 2000-2011 Pierre Sarrazin <http://sarrazip.com/>
@@ -31,8 +31,8 @@ class Controller;
 #include <flatzebra/KeyState.h>
 
 // Sound engine:
-#include <roundbeetle/Engine.h>
-#include <roundbeetle/WaveFileBuffer.h>
+#include <flatzebra/roundbeetle/SoundEngine.h>
+#include <flatzebra/roundbeetle/WaveFileBuffer.h>
 
 #include <string>
 #include <vector>
@@ -55,7 +55,8 @@ public:
                         bool mirrorHyperspace, bool useSound,
                         bool fullScreen, Controller *controller,
                         std::ostream *gameScriptStream,
-                        bool processActiveEvent);
+                        bool processActiveEvent,
+                        bool useAcceleratedRendering);
     /*  See base class.
 
         Throws a string or an integer code if an error occurs.
@@ -78,17 +79,11 @@ public:
         That object must have been constructed with operator new.
     */
 
-    virtual void processKey(SDLKey keysym, bool pressed);
-    /*  Inherited.
-    */
+    virtual void processKey(SDL_Keycode keysym, bool pressed) override;
 
-    virtual void processActivation(bool appActive);
-    /*  Inherited.
-    */
+    virtual void processActivation(bool appActive) override;
 
-    virtual bool tick();
-    /*  Inherited.
-    */
+    virtual bool tick() override;
 
 protected:
 
@@ -121,13 +116,13 @@ private:
                       const flatzebra::Couple &speed,
                       const flatzebra::Couple &collBoxPos,
                       const flatzebra::Couple &collBoxSize,
-                      roundbeetle::Engine *mixer,
+                      roundbeetle::SoundEngine *mixer,
                       float startFreq,
                       float endFreq,
                       float durationInSeconds)
         :   flatzebra::Sprite(pixmapArray, pos, speed, flatzebra::Couple(0, 0),
                               collBoxPos, collBoxSize),
-            theSoundMixer(mixer),
+            soundEngine(mixer),
             soundReq(-1),  // -1 means invalid
             linMovFreq(startFreq, endFreq, durationInSeconds)
         {
@@ -135,8 +130,8 @@ private:
 
         virtual ~SpinnerSprite()
         {
-            if (theSoundMixer != NULL && soundReq != -1)
-                theSoundMixer->stopRequest(soundReq);
+            if (soundEngine != NULL && soundReq != -1)
+                soundEngine->stopRequest(soundReq);
         }
 
         void setSoundReq(int _soundReq) { soundReq = _soundReq; }
@@ -152,7 +147,7 @@ private:
 
     private:
 
-        roundbeetle::Engine *theSoundMixer;  // may be null
+        roundbeetle::SoundEngine *soundEngine;  // may be null
         int soundReq;  // handle of the RoundBeetle sound request
         roundbeetle::LinearMovingFreq linMovFreq;  // roundbeetle thread refers to this
 
@@ -165,10 +160,10 @@ private:
     bool useSound;
 
     // SDL color values (see SDL_MapRGB()):
-    Uint32 levelColors[NUM_LEVELS + 1];  // valid indexes: 1..NUM_LEVELS
-    Uint32 whiteColor;
-    Uint32 greenColor;
-    Uint32 explosionBGColor;
+    SDL_Color levelColors[NUM_LEVELS + 1];  // valid indexes: 1..NUM_LEVELS
+    SDL_Color whiteColor;
+    SDL_Color greenColor;
+    SDL_Color explosionBGColor;
 
     bool mirrorHyperspace;
     bool useGameExtensions;
@@ -236,7 +231,7 @@ private:
     */
     enum { NUM_ROCK_HIT_SOUNDS = 2 };
 
-    roundbeetle::Engine *theSoundMixer;  // may be null
+    roundbeetle::SoundEngine *soundEngine;  // may be null
     roundbeetle::WaveFileBuffer rockHitSounds[NUM_ROCK_HIT_SOUNDS];
     roundbeetle::WaveFileBuffer playerHitSound;
     roundbeetle::WaveFileBuffer pulsarBeepSound;
@@ -253,6 +248,9 @@ private:
     /*  Implementation functions:
     */
     void fireBullet(flatzebra::Couple pos);
+    void loadPixmap(const char *filePath,
+                    flatzebra::PixmapArray &pa,
+                    size_t index);
     void initializeSprites();
     void initializeMisc(bool useSound);
     void setTimeBeforeNextPulsar();
@@ -295,6 +293,8 @@ private:
     void registerTickEnd();
     void playCadenceSound();
     void startSpinnerSound(SpinnerSprite &s);
+    void moveStars();
+    bool isBaseExploding() const;
 
 
     /*        Forbidden operations:
